<?php

namespace Drupal\commerce_stripe_checkout\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Entity\PaymentInterface;

/**
 * Defines StripeController class.
 */
class StripeController extends ControllerBase
{
    /**
     * Callback URL handling for Sendinblue API API.
     *
     * @param \Symfony\Component\HttpFoundation\Request $request
     *   Request.
     *
     * @return array
     *   Return markup for the page.
     */
    public function payment_success()
    {
        $stsess = \Drupal::request()->query->get('stsess');
        $get_stripe_stsess = $this->get_stripe_stsess();

        if ($stsess != $get_stripe_stsess) {
            return $this->redirect_homepage();
        }

        $get_stripe_payment = $this->get_stripe_payment();
        $get_stripe_order = $this->get_stripe_order();

        if ($get_stripe_order instanceof OrderInterface && $get_stripe_payment instanceof PaymentInterface) {

            commerce_stripe_checkout_change_payment_status_completed($get_stripe_payment, $get_stripe_order);

            $message = '';
            // Invoke the custom hook for success message.
            \Drupal::moduleHandler()->invokeAll('commerce_stripe_checkout_success_message', [&$message]);
            if($message == '') {
                $message = '<strong>Success!</strong> Your payment has been successfully processed.';
            }

            \Drupal::messenger()->addMessage($this->t($message));

            $this->unset_stripe_session();

            $url = Url::fromRoute('commerce_checkout.form', ['commerce_order' => $get_stripe_order->id()]);

            $response = new RedirectResponse($url->toString());
            // $response->send();
            return $response;
        } else {
            return $this->redirect_homepage();
        }
    }

    /**
     * Callback URL handling for Sendinblue API API.
     *
     * @param \Symfony\Component\HttpFoundation\Request $request
     *   Request.
     *
     * @return array
     *   Return markup for the page.
     */
    public function payment_cancel()
    {
        $stsess = \Drupal::request()->query->get('stsess');
        $get_stripe_stsess = $this->get_stripe_stsess();
        if ($stsess != $get_stripe_stsess) {
            return $this->redirect_homepage();
        }

        $get_stripe_order = $this->get_stripe_order();
        if ($get_stripe_order instanceof OrderInterface) {
            commerce_stripe_checkout_change_payment_status_cancelled($get_stripe_order);
            $this->unset_stripe_session();

            $message = '';

            // Invoke the custom hook for failure message.
            \Drupal::moduleHandler()->invokeAll('commerce_stripe_checkout_failure_message', [&$message]);

            if($message == '') {
                $message =  '<strong>Payment Canceled!</strong> Your transaction has been canceled.';
            }

            \Drupal::messenger()->addMessage($this->t($message));
        }

        return $this->redirect_homepage();
    }

    /**
     * get stripe payment
     */
    public function get_stripe_payment()
    {
        $session = \Drupal::service('session');
        $stripe_payment = $session->get('stripe_payment')??0;
        return $stripe_payment;
    }

    /**
     * Get stripe order
     */
    public function get_stripe_order()
    {
        $session = \Drupal::service('session');
        $stripe_order = $session->get('stripe_order')??0;
        return $stripe_order;
    }

    /**
     * Get stripe session key
     */
    public function get_stripe_stsess()
    {
        $session = \Drupal::service('session');
        $stsess = $session->get('stsess');
        return $stsess;
    }

    /**
     * Unset stripe session
     */
    public function unset_stripe_session()
    {
        $session = \Drupal::service('session');
        $session->set('stripe_payment', []);
        $session->set('stripe_order', []);
        $session->set('stsess', '');
    }

    /**
     * Redirect to homepage
     */
    public function redirect_homepage()
    {
        $redirect_url = Url::fromRoute('<front>', [], ['absolute' => true])->toString();
        $response = new RedirectResponse($redirect_url);
        // $response->send();
        return $response;
    }
}
